#include "config.h"
#include "screenio.h" 
#include "vdu.h"
#include <ggi/ggi.h>
#include <string.h>
#include <unistd.h>

#define FONT_LINES 8

Void (*StartText) (VOID);
Void (*MoveToTextLine) __((int line));
Void (*ClearTextLine) __((int line));
Void (*ClearScreen) (VOID);
Void (*StartGraphics) (VOID);
Void (*LoadFont) __((Char *fontname, int fontsize, double mag,
                     double hscale, double vscale));
Void (*ShowChar) __((int screenh, int screenv, Char ch));
Void (*ShowRectangle) __((int screenh, int screenv, int width, int height,
                          Char ch));
Void (*ResetVDU) (VOID);
Void (*WriteTextChar) __((Char c));
Void (*ReadTextString) __((Char *s));
Void (*ScrollRegion) __((int x1, int y1, int w, int h, int x2, int y2));
Void (*ClearRegion) __((int x1, int y1, int x2, int y2));

Static int currx, curry; /* Current position for text output */
Static ggi_pixel foreground,background;
Static ggi_pixel textforeground,textbackground;
Static ggi_color map;
ggi_visual_t *GGIvisual;

Void GGIScrollRegion(x1, y1, w, h, x2, y2)
     int x1, y1, w, h, x2, y2;
{
    ggiCopyBox(GGIvisual,x1,y1,w,h,x2,y2);
}

Void GGIClearRegion(x1, y1, x2, y2)
     int x1, y1, x2, y2;
{
    ggiSetGCForeground(GGIvisual, background);
    ggiDrawBox(GGIvisual,x1,y1,x2-x1,y2-y1);  
}

Void GGIDoNothing (VOID) { }


Void GGIShowRectangle (int screenh,int screenv,int width,int height,Char ch)
{
    ggiSetGCForeground(GGIvisual,foreground);
    ggiDrawBox(GGIvisual,screenh,screenv,width,height);
}

Void GGIMoveToTextLine (line)
int line;
{
  currx = 0;
  curry = line * FONT_LINES;
}

Void GGIClearTextLine (line)
int line;
{
    ggiSetGCForeground(GGIvisual,textbackground);
    ggiDrawBox(GGIvisual,0,(line*FONT_LINES)-(FONT_LINES),windowwd,FONT_LINES);
}

Void GGIClearScreen (VOID)
{
    ggiSetGCForeground(GGIvisual,background);
    ggiFillscreen(GGIvisual);
}

Void GGILoadFont (fontname, fontsize, mag, hscale, vscale)
Char *fontname;
int fontsize;
double mag, hscale, vscale;
{
}

Static Void PutTextChar (int h,int v,Char chr)
{
  ggiSetGCForeground(GGIvisual,textforeground);
  ggiSetGCBackground(GGIvisual,textbackground);
  ggiPutc(GGIvisual,h, v - FONT_LINES, chr);

} /* PutTextChar */
 
Void GGIWriteTextChar (Char ch)
{
  switch (ch) {
  case '\n':
    currx = 0;
    curry = curry + FONT_LINES;
    break;
  case '\b':
  case 127:
    break;
  default:
    PutTextChar(currx, curry, ch);
    currx += 8;
    break;
  } 
}


Void GGIReadTextString (s)
Char *s;
{
  Char ch;
  int i = 0;

  while (1) {
    ch=ggiGetc(GGIvisual);
    switch (ch) {
    case '\n':
      s[i] = '\0';      
      return;
    case '\b':
    case 127:
      if (i > 0) {
	currx -= 8;
        ggiSetGCForeground(GGIvisual,background);
	ggiDrawBox(GGIvisual,currx, curry - FONT_LINES, 8, FONT_LINES);
	i--;
      }
      break;
    default:
      s[i++] = ch;
      break;
    }
  }
}

Void GGIShowChar (int screenh,int screenv,Char ch)
{
  Char newch;   /* = TeXtoASCII[ch] */
  int v;

  v = screenv + 5; /* Match characters with TeX baseline */
  newch = TeXtoASCII[ch - NUL];   /* convert TeX ch to ASCII */
  if (newch != '?') {
    PutTextChar(screenh, v, newch);
  } else {
    switch (ch) {
    case 0x00: /* Gamma */
      PutTextChar(screenh, v, 226);
      break;
    case 0x02: /* Theta */
      PutTextChar(screenh, v, 233);
      break;
    case 0x05: /* Pi */
      PutTextChar(screenh, v, 227);
      break;
    case 0x06: /* Sigma */
      PutTextChar(screenh, v, 228);
      break;
    case 0x08: /* Phi */
      PutTextChar(screenh, v, 232);
      break;
    case 0x0a: /* Omega */
      PutTextChar(screenh, v, 233);
      break;
    case 0x0b: /* ff */
      PutTextChar(screenh, v, 'f');
      PutTextChar(screenh+8, v, 'f');
      break;
    case 0x0c: /* fi */
      PutTextChar(screenh, v, 'f');
      PutTextChar(screenh+8, v, 'i');
      break;
    case 0x0d: /* fl */
      PutTextChar(screenh, v, 'f');
      PutTextChar(screenh+8, v, 'l');
      break;
    case 0x0e: /* ffi */
      PutTextChar(screenh, v, 'f');
      PutTextChar(screenh+8, v, 'f');
      PutTextChar(screenh+8, v, 'i');
      break;
    case 0x0f: /* ffl */
      PutTextChar(screenh, v, 'f');
      PutTextChar(screenh+8, v, 'f');
      PutTextChar(screenh+16, v, 'l');
      break;
    case 0x17: /* Degree mark */
      PutTextChar(screenh, v, 248);
      break;
    case 0x19: /* German sharp S */
      PutTextChar(screenh, v, 225);
      break;
    case 0x1a: /* ae */
      PutTextChar(screenh, v, 145);
      break;
    case 0x1b: /* oe */
      PutTextChar(screenh, v, 'o');
      PutTextChar(screenh+8, v, 'e');
      break;
    case 0x1d: /* AE */
      PutTextChar(screenh, v, 146);
      break;
    case 0x1e: /* OE */
      PutTextChar(screenh, v, 'O');
      PutTextChar(screenh+8, v, 'E');
      break;
    case 0x3c: /* ! */
      PutTextChar(screenh, v, 173);
      break;
    case 0x3e: /* ? */
      PutTextChar(screenh, v, 168);
      break;
    default:
      PutTextChar(screenh+8, v, '?');
      break;
    }
  }
}

Void GGIResetVDU (VOID)
{
    ggiClose(GGIvisual);
    ggiExit();
}

Void InitGGI (VOID)
{
    ggi_mode GGImode;
    if(ggiInit()) {}
    if(NULL==(GGIvisual=ggiOpen(NULL))) ggiExit();
    if(ggiSetSimpleMode(GGIvisual,GGI_AUTO,GGI_AUTO,GGI_AUTO,GT_AUTO)) ggiExit();
    ggiSetFlags(GGIvisual, GGIFLAG_ASYNC);
    ggiGetMode(GGIvisual,&GGImode);
    windowwd=GGImode.virt.x;
    windowht=GGImode.virt.y;
    map.r=0xFFFF; map.g=0xFFFF; map.b=0xFFFF; background=ggiMapColor(GGIvisual,&map);
    map.r=0x0000; map.g=0x0000; map.b=0x0000; foreground=ggiMapColor(GGIvisual,&map);
    map.r=0x0000; map.g=0x0000; map.b=0xFFFF;
    textbackground=ggiMapColor(GGIvisual,&map);
    map.r=0xFFFF; map.g=0xFFFF; map.b=0x0000;
    textforeground=ggiMapColor(GGIvisual,&map);

  /* The dialogue region is the top 4 lines.
     The window region is the remaining area of the screen.
     */
  DVIstatusl = 1;
  windowstatusl = 2;
  messagel = 3;
  commandl = 4;
  bottoml = 24;
  windowh = 0;
  windowv = (FONT_LINES * 4);
  
  MoveToTextLine = GGIMoveToTextLine;
  ClearTextLine  = GGIClearTextLine;
  ClearScreen    = GGIClearScreen;
  StartText      = GGIDoNothing;
  StartGraphics  = GGIDoNothing;
  LoadFont       = GGILoadFont;
  ShowChar       = GGIShowChar;
  ShowRectangle  = GGIShowRectangle;
  ResetVDU       = GGIResetVDU;
  WriteTextChar  = GGIWriteTextChar;
  ReadTextString = GGIReadTextString;
  ScrollRegion   = GGIScrollRegion;
  ClearRegion    = GGIClearRegion;
  
  currx = 0;
  curry = 0;
}
